@@ -6,9 +6,10 @@ a vector is a one dimensional numpy array with 3 elements: x, y, z.
 some functions accept a tuple or list of 3 elements as an alternative to a numpy array.
 """
 
-version = '30th June 2022'
+version = '28th July 2022'
 
 import logging
+from typing import Tuple, Optional
 
 log = logging.getLogger(__name__)
 
@@ -16,6 +17,7 @@ import math as maths
 import numpy as np
 import numba  # type: ignore
 from numba import njit
+import multiprocessing as m_p
 
 
 def radians_from_degrees(deg):
@@ -288,7 +290,7 @@ def degrees_difference(a, b):
 
 
 def rotation_matrix_3d_axial(axis, angle):
-    """Returns a rotation matrix which will rotate points about axis (0, 1, or 2) by angle in degrees."""
+    """Returns a rotation matrix which will rotate points about axis (0: x, 1: y, or 2: z) by angle in degrees."""
 
     axis_a = (axis + 1) % 3
     axis_b = (axis_a + 1) % 3
@@ -304,6 +306,14 @@ def rotation_matrix_3d_axial(axis, angle):
     return matrix
 
 
+def no_rotation_matrix():
+    """Returns a rotation matrix which will not move points."""
+    matrix = np.zeros((3, 3))
+    for axis in range(3):
+        matrix[axis, axis] = 1.0
+    return matrix
+
+
 def rotation_3d_matrix(xzy_axis_angles):
     """Returns a rotation matrix which will rotate points about 3 axes by angles in degrees."""
 
@@ -379,6 +389,18 @@ def tilt_3d_matrix(azimuth, dip):
     return matrix
 
 
+def rotation_matrix_3d_vector(v):
+    """Returns a rotation matrix which will rotate points by inclination and azimuth of vector.
+
+    note:
+       the returned matrix will map a positive z axis vector onto v
+    """
+
+    m = tilt_3d_matrix(azimuth(v), inclination(v))
+    m[:2, :] = -m[:2, :]  # todo: should this change be in the tilt matrix?
+    return m
+
+
 def tilt_points(pivot_xyz, azimuth, dip, points):
     """Modifies array of xyz points in situ to apply dip in direction of azimuth, about pivot point."""
 
@@ -558,39 +580,372 @@ def point_in_polygon(x, y, polygon):
 
 
 @njit
-def points_in_polygon(points, polygon):
+def point_in_triangle(x, y, triangle):
+    """Calculates if a point in within a triangle in 2D.
+
+    Args:
+        x (float): the point's x-coordinate.
+        y (float): the point's y-coordinate.
+        triangle (np.ndarray): array of the triangles's vertices in 2D, of shape (3, 2)
+
+    Returns:
+        inside (bool): True if point is within the polygon, False otherwise.
+    """
+    p0x = triangle[0, 0]
+    p1x = triangle[1, 0]
+    p2x = triangle[2, 0]
+    min_p01x = min(p0x, p1x)
+    if x < min(min_p01x, p2x):
+        return False
+    max_p01x = max(p0x, p1x)
+    if x > max(max_p01x, p2x):
+        return False
+    p0y = triangle[0, 1]
+    p1y = triangle[1, 1]
+    p2y = triangle[2, 1]
+    min_p01y = min(p0y, p1y)
+    if y < min(min_p01y, p2y):
+        return False
+    max_p01y = max(p0y, p1y)
+    if y > max(max_p01y, p2y):
+        return False
+    inside = False
+    xints = 0.0
+    if y > min_p01y:
+        if y <= max_p01y:
+            if x <= max_p01x:
+                if p0x == p1x:
+                    inside = True
+                else:
+                    if p0y == p1y:
+                        inside = True
+                    else:
+                        xints = (y - p0y) * (p1x - p0x) / (p1y - p0y) + p0x
+                        if x <= xints:
+                            inside = True
+    if y > min(p1y, p2y):
+        if y <= max(p1y, p2y):
+            if x <= max(p1x, p2x):
+                if p1x == p2x:
+                    inside = not inside
+                else:
+                    if p1y == p2y:
+                        inside = not inside
+                    else:
+                        xints = (y - p1y) * (p2x - p1x) / (p2y - p1y) + p1x
+                        if x <= xints:
+                            inside = not inside
+    if y > min(p2y, p0y):
+        if y <= max(p2y, p0y):
+            if x <= max(p2x, p0x):
+                if p2x == p0x:
+                    inside = not inside
+                else:
+                    if p2y == p0y:
+                        inside = not inside
+                    else:
+                        xints = (y - p2y) * (p0x - p2x) / (p0y - p2y) + p2x
+                        if x <= xints:
+                            inside = not inside
+    return inside
+
+
+@njit
+def points_in_polygon(points: np.ndarray, polygon: np.ndarray, points_xlen: int, polygon_num: int = 0) -> np.ndarray:
     """Calculates which points are within a polygon in 2D.
-    
+
     Args:
         points (np.ndarray): array of the points in 2D.
         polygon (np.ndarray): array of the polygon's vertices in 2D.
-    
+        points_xlen (int): the number of unique x coordinates.
+        polygon_num (int): the polygon number, default is 0.
+
     Returns:
-        polygon_points (np.ndarray): boolean array of which points are within the polygon.
+        polygon_points (np.ndarray): 2D array containing only the points within the polygon,
+            with each row being the polygon number, points y index, and points x index.
     """
-    polygon_points = np.empty(len(points), dtype = numba.boolean)
-    for point_num in numba.prange(0, len(polygon_points)):
-        polygon_points[point_num] = point_in_polygon(points[point_num, 0], points[point_num, 1], polygon)
+    polygon_points = np.empty((0, 3), dtype = numba.int32)
+    for point_num in numba.prange(len(points)):
+        p = point_in_polygon(points[point_num, 0], points[point_num, 1], polygon)
+        if p is True:
+            yi, xi = divmod(point_num, points_xlen)
+            polygon_points = np.append(polygon_points, np.array([[polygon_num, yi, xi]], dtype = numba.int32), axis = 0)
+
     return polygon_points
 
 
-@njit(parallel = True)
-def points_in_polygons(points: np.ndarray, polygons: np.ndarray) -> np.ndarray:
+@njit
+def points_in_triangle(points: np.ndarray, triangle: np.ndarray, points_xlen: int, triangle_num: int = 0) -> np.ndarray:
+    """Calculates which points are within a triangle in 2D.
+
+    Args:
+        points (np.ndarray): array of the points in 2D.
+        triangle (np.ndarray): array of the triangle's vertices in 2D, shape (3, 2).
+        points_xlen (int): the number of unique x coordinates.
+        triangle_num (int): the triangle number, default is 0.
+
+    Returns:
+        triangle_points (np.ndarray): 2D array containing only the points within the triangle,
+            with each row being the triangle number, points y index, and points x index.
+    """
+    triangle_points = np.empty((0, 3), dtype = numba.int32)
+    for point_num in numba.prange(len(points)):
+        p = point_in_triangle(points[point_num, 0], points[point_num, 1], triangle)
+        if p is True:
+            yi, xi = divmod(point_num, points_xlen)
+            triangle_points = np.append(triangle_points,
+                                        np.array([[triangle_num, yi, xi]], dtype = numba.int32),
+                                        axis = 0)
+
+    return triangle_points
+
+
+@njit
+def mesh_points_in_triangle(triangle: np.ndarray,
+                            points_xlen: int,
+                            points_ylen: int,
+                            triangle_num: int = 0) -> np.ndarray:
+    """Calculates which implicit mesh points are within a triangle in 2D for normalised triangle.
+
+    Args:
+        triangle (np.ndarray): array of the triangle's vertices in 2D, shape (3, 2).
+        points_xlen (int): the number of unique x coordinates, starting at 0.0, spacing 1.0.
+        points_ylen (int): the number of unique y coordinates, starting at 0.0, spacing 1.0.
+        triangle_num (int): the triangle number, default is 0.
+
+    Returns:
+        triangle_points (np.ndarray): 2D array containing only the points within the triangle,
+            with each row being the triangle number, points y index, and points x index.
+    """
+    triangle_points = np.empty((0, 3), dtype = numba.int32)
+    y = 0.0
+    for yi in numba.prange(points_ylen):
+        x = 0.0
+        for xi in numba.prange(points_xlen):
+            p = point_in_triangle(x, y, triangle)
+            if p is True:
+                triangle_points = np.append(triangle_points,
+                                            np.array([[triangle_num, yi, xi]], dtype = numba.int32),
+                                            axis = 0)
+            x += 1.0
+        y += 1.0
+    return triangle_points
+
+
+def points_in_polygons_parallel(points: np.ndarray, polygons: np.ndarray, points_xlen: int) -> np.ndarray:
     """Calculates which points are within which polygons in 2D.
-    
+
     Args:
         points (np.ndarray): array of the points in 2D.
         polygons (np.ndarray): array of each polygons' vertices in 2D.
-    
+        points_xlen (int): the number of unique x coordinates.
+
     Returns:
-        polygons_points (np.ndarray): boolean array of which points are within each polygon.
+        polygons_points (np.ndarray): 2D array containing only the points within each polygon,
+            with each row being the polygon number, points y index, and points x index.
     """
-    polygons_points = np.empty((len(polygons), len(points)), dtype = numba.boolean)
-    for polygon_num in numba.prange(0, len(polygons)):
-        polygons_points[polygon_num] = points_in_polygon(points, polygons[polygon_num])
+    args = [(points, polygons[polygon_num], points_xlen, polygon_num) for polygon_num in range(len(polygons))]
+    with m_p.Pool() as p:
+        r = p.starmap(points_in_polygon, args)
+
+    return np.vstack(r)
+
+
+@njit
+def points_in_polygons(points: np.ndarray, polygons: np.ndarray, points_xlen: int) -> np.ndarray:
+    """Calculates which points are within which polygons in 2D.
+
+    Args:
+        points (np.ndarray): array of the points in 2D.
+        polygons (np.ndarray): array of each polygons' vertices in 2D.
+        points_xlen (int): the number of unique x coordinates.
+
+    Returns:
+        polygons_points (np.ndarray): 2D array (list-like) containing only the points within each polygon,
+            with each row being the polygon number, points y index, and points x index.
+    """
+    polygons_points = np.empty((0, 3), dtype = numba.int32)
+    for polygon_num in numba.prange(len(polygons)):
+        polygon_points = points_in_polygon(points, polygons[polygon_num], points_xlen, polygon_num)
+        polygons_points = np.append(polygons_points, polygon_points, axis = 0)
+
     return polygons_points
 
 
+@njit
+def points_in_triangles_njit(points: np.ndarray, triangles: np.ndarray, points_xlen: int) -> np.ndarray:
+    """Calculates which points are within which triangles in 2D.
+
+    Args:
+        points (np.ndarray): array of the points in 2D.
+        triangles (np.ndarray): array of each triangles' vertices in 2D, shape (N, 3, 2).
+        points_xlen (int): the number of unique x coordinates.
+
+    Returns:
+        triangles_points (np.ndarray): 2D array (list-like) containing only the points within each triangle,
+            with each row being the triangle number, points y index, and points x index.
+    """
+    triangles_points = np.empty((0, 3), dtype = numba.int32)
+    for triangle_num in numba.prange(len(triangles)):
+        triangle_points = points_in_triangle(points, triangles[triangle_num], points_xlen, triangle_num)
+        triangles_points = np.append(triangles_points, triangle_points, axis = 0)
+
+    return triangles_points
+
+
+@njit
+def meshgrid(x: np.ndarray, y: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    """Returns coordinate matrices from coordinate vectors x and y.
+
+    Args:
+        x (np.ndarray): 1d array of x coordinates.
+        y (np.ndarray): 1d array of y coordinates.
+
+    Returns:
+        Tuple containing:
+
+            - xx (np.ndarray): the elements of x repeated to fill the matrix along the first dimension.
+            - yy (np.ndarray): the elements of y repeated to fill the matrix along the second dimension.
+    """
+    xx = np.empty(shape = (y.size, x.size), dtype = x.dtype)
+    yy = np.empty(shape = (y.size, x.size), dtype = y.dtype)
+    for i in range(y.size):
+        for j in range(x.size):
+            xx[i, j] = x[j]
+            yy[i, j] = y[i]
+
+    return xx, yy
+
+
+def points_in_triangles_aligned(nx: int, ny: int, dx: float, dy: float, triangles: np.ndarray) -> np.ndarray:
+    """Calculates which points are within which triangles in 2D for a regular mesh of aligned points.
+
+    arguments:
+        nx (int): number of points in x axis
+        ny (int): number of points in y axis
+        dx (float): spacing of points in x axis (first point is at half dx)
+        dy (float): spacing of points in y axis (first point is at half dy)
+        triangles (np.ndarray): float array of each triangles' vertices in 2D, shape (N, 3, 2).
+        points_xlen (int): the number of unique x coordinates.
+
+    returns:
+        triangles_points (np.ndarray): 2D array (list-like) containing only the points within each triangle,
+            with each row being the triangle number, points y index, and points x index.
+    """
+    triangles_points = np.empty((0, 3), dtype = np.int32)
+    dx_dy = np.expand_dims(np.array([dx, dy], dtype = np.float32), axis = 0)
+    # for triangle_num in numba.prange(len(triangles)):
+    for triangle_num in range(len(triangles)):
+        tp = (triangles[triangle_num] / dx_dy) - 0.5
+        min_tpx = max(maths.ceil(min(tp[0, 0], tp[1, 0], tp[2, 0])), 0)
+        max_tpx = min(maths.floor(max(tp[0, 0], tp[1, 0], tp[2, 0])), nx - 1)
+        if max_tpx < min_tpx:
+            continue
+        min_tpy = max(maths.ceil(min(tp[0, 1], tp[1, 1], tp[2, 1])), 0)
+        max_tpy = min(maths.floor(max(tp[0, 1], tp[1, 1], tp[2, 1])), ny - 1)
+        if max_tpy < min_tpy:
+            continue
+        ntpx = max_tpx - min_tpx + 1
+        ntpy = max_tpy - min_tpy + 1
+        # x = np.linspace(float(min_tpx), float(max_tpx), ntpx)
+        # y = np.linspace(float(min_tpy), float(max_tpy), ntpy)
+        # p = np.stack(meshgrid(x, y), axis = -1).reshape((-1, 2))
+        # triangle_points = points_in_triangle(p, tp, ntpx, triangle_num)
+        tp[:, 0] -= float(min_tpx)
+        tp[:, 1] -= float(min_tpy)
+        triangle_points = mesh_points_in_triangle(tp, ntpx, ntpy, triangle_num)
+        triangle_points[:, 1] += min_tpy
+        triangle_points[:, 2] += min_tpx
+        triangles_points = np.append(triangles_points, triangle_points, axis = 0)
+
+    return triangles_points
+
+
+@njit
+def triangle_box(triangle: np.ndarray) -> Tuple[float, float, float, float]:
+    """Finds the minimum and maximum x and y values of a single traingle.
+
+    Args:
+        triangle (np.ndarray): array of the traingle's vertices' x and y coordinates.
+
+    Returns:
+        Tuple containing:
+
+            - (float): minimum x value.
+            - (float): maximum x value.
+            - (float): minimum y value.
+            - (float): maximum y value.
+    """
+    x_values = triangle[:, 0]
+    y_values = triangle[:, 1]
+    return min(x_values), max(x_values), min(y_values), max(y_values)
+
+
+@njit
+def vertical_intercept(x: float, x_values: np.ndarray, y_values: np.ndarray) -> Optional[float]:
+    """Finds the y value of a straight line between two points at a given x.
+    
+    If the x value given is not within the x values of the points, returns None.
+
+    Args:
+        x (float): x value at which to determine the y value.
+        x_values (np.ndarray): the x coordinates of point 1 and point 2.
+        y_values (np.ndarray): the y coordinates of point 1 and point 2.
+
+    Returns:
+        y (Optional[float]): y value of the straight line between point 1 and point 2,
+            evaluated at x. If x is outside the x_values range, y is None.
+    """
+    y = None
+    if x >= min(x_values) and x <= max(x_values):
+        m = (y_values[1] - y_values[0]) / (x_values[1] - x_values[0])
+        c = y_values[1] - m * x_values[1]
+        y = m * x + c
+    return y
+
+
+@njit
+def points_in_triangles_aligned_optimised(nx: int, ny: int, dx: float, dy: float, triangles: np.ndarray) -> np.ndarray:
+    """Calculates which points are within which triangles in 2D for a regular mesh of aligned points.
+
+    Args:
+        nx (int): number of points in x axis
+        ny (int): number of points in y axis
+        dx (float): spacing of points in x axis (first point is at half dx)
+        dy (float): spacing of points in y axis (first point is at half dy)
+        triangles (np.ndarray): float array of each triangles' vertices in 2D, shape (N, 3, 2).
+        points_xlen (int): the number of unique x coordinates.
+
+    Returns:
+        triangles_points (np.ndarray): 2D array (list-like) containing only the points within each triangle,
+            with each row being the triangle number, points y index, and points x index.
+    """
+    grid_x = np.arange(dx / 2, dx / 2 + dx * nx, dx)
+    grid_y = np.arange(dy / 2, dy / 2 + dy * ny, dy)
+    triangles_points_list = []
+    for triangle_num in range(len(triangles)):
+        triangle = triangles[triangle_num]
+        min_x, max_x, min_y, max_y = triangle_box(triangle)
+        x_values = grid_x[np.logical_and(grid_x > min_x, grid_x < max_x)]
+        y_values = grid_y[np.logical_and(grid_y > min_y, grid_y < max_y)]
+        for x in x_values:
+            ys = []
+            ys.append(vertical_intercept(x, triangle[1:, 0], triangle[1:, 1]))
+            ys.append(vertical_intercept(x, triangle[:2, 0], triangle[:2, 1]))
+            ys.append(vertical_intercept(x, triangle[::2, 0], triangle[::2, 1]))
+            ys = [x for x in ys if x is not None]
+            valid_y = y_values[np.logical_and(y_values >= min(ys), y_values <= max(ys))]
+            x_idx = int(x / dx - 0.5)
+            triangles_points_list.extend([[triangle_num, int(y / dy - 0.5), x_idx] for y in valid_y])
+
+    if len(triangles_points_list) == 0:
+        triangles_points = np.empty((0, 3), dtype = np.int32)
+        return triangles_points
+
+    triangles_points = np.array(triangles_points_list, dtype = np.int32)
+    return triangles_points
+
+
 def triangle_normal_vector(p3):
     """For a triangle in 3D space, defined by 3 vertex points, returns a unit vector normal to the plane of the triangle."""
 
