@@ -53,7 +53,8 @@ class EarthModelInterpretation(BaseResqpy):
         if self is other or bu.matching_uuids(self.uuid, other.uuid):
             return True
         if self.organization_feature is not None:
-            if not self.organization_feature.is_equivalent(other.organization_feature):
+            if not self.organization_feature.is_equivalent(other.organization_feature,
+                                                           check_extra_metadata = check_extra_metadata):
                 return False
         elif other.organization_feature is not None:
             return False
