@@ -8,6 +8,7 @@ import logging
 
 log = logging.getLogger(__name__)
 
+import math as maths
 import numpy as np
 
 import resqpy.olio.uuid as bu
@@ -64,6 +65,8 @@ def _create_xml_get_basics(collection, discrete, points, const_value, facet_type
 
 def _create_xml_property_kind(collection, p_node, find_local_property_kinds, property_kind, uom, discrete,
                               property_kind_uuid):
+    if property_kind == 'permeability rock':
+        property_kind = 'rock permeability'
     p_kind_node = rqet.SubElement(p_node, ns['resqml2'] + 'PropertyKind')
     p_kind_node.text = rqet.null_xml_text
     if find_local_property_kinds and property_kind not in supported_property_kind_list:
@@ -81,6 +84,7 @@ def _create_xml_property_kind(collection, p_node, find_local_property_kinds, pro
                                          property_kind_uuid,
                                          content_type = 'obj_PropertyKind',
                                          root = p_kind_node)
+    return property_kind_uuid
 
 
 def _create_xml_patch_node(collection, p_node, points, const_value, indexable_element, direction, p_uuid, ext_uuid,
@@ -105,19 +109,25 @@ def _create_xml_patch_node(collection, p_node, points, const_value, indexable_el
 
 
 def _create_xml_property_min_max(collection, property_array, const_value, discrete, add_min_max, p_node, min_value,
-                                 max_value):
-    if add_min_max:
+                                 max_value, categorical, null_value):
+    if add_min_max and not categorical:
         # todo: use active cell mask on numpy min and max operations; exclude null values on discrete min max
         min_value, max_value = pcga._get_property_array_min_max_value(collection, property_array, const_value, discrete,
-                                                                      min_value, max_value)
+                                                                      min_value, max_value, categorical, null_value)
         if min_value is not None:
             min_node = rqet.SubElement(p_node, ns['resqml2'] + 'MinimumValue')
             min_node.set(ns['xsi'] + 'type', ns['xsd'] + collection.xsd_type)
-            min_node.text = str(min_value)
+            if discrete:
+                min_node.text = str(maths.floor(min_value))
+            else:
+                min_node.text = str(min_value)
         if max_value is not None:
             max_node = rqet.SubElement(p_node, ns['resqml2'] + 'MaximumValue')
             max_node.set(ns['xsi'] + 'type', ns['xsd'] + collection.xsd_type)
-            max_node.text = str(max_value)
+            if discrete:
+                max_node.text = str(maths.ceil(max_value))
+            else:
+                max_node.text = str(max_value)
 
 
 def _create_xml_lookup_node(collection, p_node, string_lookup_uuid):
