@@ -1,4 +1,7 @@
 """Module containing common methods for properties"""
+
+# Nexus is a trademark of Halliburton
+
 import warnings
 import resqpy.property
 
@@ -21,10 +24,10 @@ import resqpy.weights_and_measures as bwam
 # see property_kind_and_facet_from_keyword() for simulator keyword to property kind and facet mapping
 
 supported_property_kind_list = [
-    'code', 'index', 'depth', 'rock volume', 'pore volume', 'volume', 'thickness', 'length', 'cell length',
-    'net to gross ratio', 'porosity', 'permeability thickness', 'permeability length', 'permeability rock',
-    'rock permeability', 'fluid volume', 'transmissibility', 'pressure', 'saturation', 'solution gas-oil ratio',
-    'vapor oil-gas ratio', 'property multiplier', 'thermodynamic temperature', 'continuous', 'discrete', 'categorical'
+    'continuous', 'discrete', 'categorical', 'code', 'index', 'depth', 'rock volume', 'pore volume', 'volume',
+    'thickness', 'length', 'cell length', 'area', 'net to gross ratio', 'porosity', 'permeability thickness',
+    'permeability length', 'permeability rock', 'rock permeability', 'fluid volume', 'transmissibility', 'pressure',
+    'saturation', 'solution gas-oil ratio', 'vapor oil-gas ratio', 'property multiplier', 'thermodynamic temperature'
 ]
 
 supported_local_property_kind_list = [
@@ -173,6 +176,9 @@ def property_kind_and_facet_from_keyword(keyword):
         property_kind = 'active'  # local property kind, see RESQML (2.0.1) usage guide, section 11.17
     elif lk[0] == 'i' or lk.startswith('reg') or lk.startswith('creg'):
         property_kind = 'region initialization'  # local property kind, see RESQML (2.0.1) usage guide, section 11.18
+        if lk[0] == 'i':
+            facet_type = 'what'
+            facet = lk[1:].lower()
     return property_kind, facet_type, facet
 
 
@@ -342,6 +348,8 @@ def _crs_m_or_ft(crs_node):  # NB. models not-so-rarely use metres for xy and fe
         return 'm'
     if xy_units == 'ft' and z_units == 'ft':
         return 'ft'
+    if xy_units == 'cm' and z_units == 'cm':
+        return 'cm'
     return None
 
 
@@ -361,7 +369,8 @@ def guess_uom(property_kind, minimum, maximum, support, facet_type = None, facet
     returns:
        a valid resqml unit of measure (uom) for the property_kind, or None
 
-    note:
+    notes:
+       this function is tailored towards Nexus unit systems;
        the resqml standard allows a property to have any number of facets; however,
        this module currently only supports zero or one facet per property
     """
@@ -378,7 +387,7 @@ def guess_uom(property_kind, minimum, maximum, support, facet_type = None, facet
     if property_kind == 'permeability rock' or property_kind == 'rock permeability':
         return 'mD'
     if property_kind in ['permeability thickness', 'permeability length']:
-        return _guess_uom_permeability(property_kind, crs_node, from_crs)
+        return _guess_uom_permeability_thickness(property_kind, crs_node, from_crs)
     if property_kind.endswith('transmissibility'):
         return _guess_uom_transmissibility(from_crs)
     if property_kind == 'pressure':
@@ -414,28 +423,23 @@ def _guess_uom_volume(property_kind, from_crs, facet_type, facet):
                 return '1000 ft3'  # todo: check units output by nexus for GIP
             else:
                 return 'bbl'  # todo: check whether nexus uses 10^3 or 10^6 units
+        if from_crs == 'cm':
+            return 'cm3'
         return None
     if from_crs is None:
         return None
     if from_crs == 'ft' and property_kind == 'pore volume':
         return 'bbl'  # seems to be Nexus 'ENGLISH' uom for pv out
-    return from_crs + '3'  # ie. m3 or ft3
+    return from_crs + '3'  # ie. m3 or ft3 or cm3
 
 
-def _guess_uom_permeability(property_kind, crs_node, from_crs):
-    if 'thickness' in property_kind:
-        z_units = rqet.find_tag(crs_node, 'VerticalUom').text.lower()
-        if z_units == 'm':
-            return 'mD.m'
-        if z_units == 'ft':
-            return 'mD.ft'
-        return None
-    else:
-        if from_crs is not None:
-            return f'mD.{from_crs}'
-        else:
-            return None
-        return None
+def _guess_uom_permeability_thickness(property_kind, crs_node, from_crs):
+    z_units = rqet.find_tag(crs_node, 'VerticalUom').text.lower()
+    if z_units in ['m', 'cm']:  # note: mD.cm is not a valid RESQML uom
+        return 'mD.m'
+    if z_units == 'ft':
+        return 'mD.ft'
+    return None
 
 
 def _guess_uom_ntg_por_sat(maximum, from_crs):
@@ -449,6 +453,8 @@ def _guess_uom_ntg_por_sat(maximum, from_crs):
         return 'm3/m3'
     if from_crs == 'ft':
         return 'ft3/ft3'
+    if from_crs == 'cm':
+        return 'cm3/cm3'
     return 'Euc'
 
 
@@ -458,13 +464,15 @@ def _guess_uom_transmissibility(from_crs):
         return 'm3.cP/(kPa.d)'  # NB: might actually be m3/(psi.d) or m3/(bar.d)
     if from_crs == 'ft':
         return 'bbl.cP/(psi.d)'  # gamble on barrels per day per psi; could be ft3/(psi.d)
+    if from_crs == 'cm':
+        return 'cm3.cP/(psi.d)'  # complete guess, though transmissibility probably not used in lab scale models
     return None
 
 
 def _guess_uom_pressure(from_crs, maximum):
     if from_crs == 'm':
         return 'kPa'  # NB: might actually be psi or bar
-    if from_crs == 'ft':
+    if from_crs in ['ft', 'cm']:  # note: Nexus uses psi for lab pressure units
         return 'psi'
     if maximum is not None:
         max_real = float(maximum)
@@ -487,6 +495,8 @@ def _guess_uom_gor_ogr(property_kind, from_crs):
             return '1000 ft3/bbl'
         else:
             return '0.001 bbl/ft3'
+    if from_crs == 'cm':
+        return 'cm3/cm3'
     return None
 
 
@@ -510,7 +520,9 @@ def selective_version_of_collection(
         string_lookup_uuid = None,
         categorical = None,
         title = None,
-        related_uuid = None):
+        title_mode = None,
+        related_uuid = None,
+        const_value = None):
     """Returns a new PropertyCollection with those parts which match all arguments that are not None.
 
     arguments:
@@ -521,9 +533,12 @@ def selective_version_of_collection(
     Other optional arguments:
     realization, support_uuid, grid, uuid, continuous, points, count, indexable, property_kind, facet_type, facet,
     citation_title, time_series_uuid, time_index, uom, string_lookup_uuid, categorical, title, related_uuid:
+    title_mode (str, optional): if present, one of 'is', 'starts', 'ends', 'contains', 'is not',
+        'does not start', 'does not end', 'does not contain'; None is the same as 'is'
 
     for each of these arguments: if None, then all members of collection pass this filter;
     if not None then only those members with the given value pass this filter;
+    special values: '*' any non-None value passes; 'none' only None passes
     finally, the filters for all the attributes must be passed for a given member
     to be included in the returned collection; title is a synonym for the citation_title argument;
     related_uuid will pass if a soft relationship exists
@@ -560,12 +575,14 @@ def selective_version_of_collection(
                                                          facet_type = facet_type,
                                                          facet = facet,
                                                          citation_title = citation_title,
+                                                         citation_title_match_mode = title_mode,
                                                          time_series_uuid = time_series_uuid,
                                                          time_index = time_index,
                                                          uom = uom,
                                                          string_lookup_uuid = string_lookup_uuid,
                                                          categorical = categorical,
-                                                         related_uuid = related_uuid)
+                                                         related_uuid = related_uuid,
+                                                         const_value = const_value)
     return view
 
 
@@ -724,3 +741,11 @@ def write_hdf5_and_create_xml_for_active_property(model,
                                                  realization = realization,
                                                  find_local_property_kind = True)
     return active.uuid
+
+
+def check_and_warn_property_kind(pk, activity):
+    """Check property kind and warn if one of three main abstract kinds."""
+    if pk in ['continuous', 'discrete', 'categorical']:
+        warnings.warn(
+            f"abstract property kind '{pk}', whilst {activity}, will be or have been replaced by local property kind")
+        # raise ValueError
