@@ -457,7 +457,7 @@ def import_nexus(
                                                                          uom = None,
                                                                          time_index = None,
                                                                          null_value = -1,
-                                                                         property_kind = 'discrete')
+                                                                         property_kind = 'cell index')
         log.info('writing ' + str(len(prop_import_collection.imported_list)) + ' properties to hdf5 file ' +
                  resqml_file_root + '.h5')
     elif not ensemble_case_dirs_root:
@@ -522,7 +522,7 @@ def import_nexus(
                 # for each property for this timestep, cache array and add to recur prop import collection for this time step
                 if recur_prop_list:
                     for keyword in recur_prop_list:
-                        if not keyword or not keyword.isalnum():
+                        if vdb.bad_keyword(keyword):
                             continue
                         prop_kind, facet_type, facet = rp.property_kind_and_facet_from_keyword(keyword)
                         step_import_collection.import_vdb_recurrent_property_to_cache(
